use actix_web::{http::header::LOCATION, web, HttpResponse, Responder};
use garde::Validate;
use sqlx::PgPool;

use crate::{
    endpoints::user::NewOrEditUserForm,
    mail::Mailer,
    models::{Function, Registration, Role, User, UserChangeset},
    utils::ApplicationError,
};

#[actix_web::post("/users/new")]
pub async fn post_new(
    user: web::ReqData<User>,
    pool: web::Data<PgPool>,
    form: web::Form<NewOrEditUserForm>,
    mailer: web::Data<Mailer>,
) -> Result<impl Responder, ApplicationError> {
    if user.role != Role::AreaManager && user.role != Role::Admin {
        return Err(ApplicationError::Unauthorized);
    }

    let mut area_id = user.area_id;

    if user.role == Role::Admin && form.area.is_some() {
        area_id = form.area.unwrap();
    }

    let role = Role::try_from(form.role)?;
    if role == Role::Admin && user.role != Role::Admin {
        return Err(ApplicationError::Unauthorized);
    }

    let mut functions = Vec::with_capacity(3);

    if form.is_posten.unwrap_or(false) {
        functions.push(Function::Posten);
    }

    if form.is_wachhabender.unwrap_or(false) {
        functions.push(Function::Wachhabender);
    }

    if form.is_fuehrungsassistent.unwrap_or(false) {
        functions.push(Function::Fuehrungsassistent);
    }

    let changeset = UserChangeset {
        name: form.name.clone(),
        email: form.email.clone(),
        role,
        functions,
        area_id,
    };

    if let Err(e) = changeset.validate() {
        return Ok(HttpResponse::BadRequest().body(e.to_string()));
    };

    let id = User::create(pool.get_ref(), changeset).await?;

    let registration = Registration::insert_new_for_user(pool.get_ref(), id).await?;
    let new_user = User::read_by_id(pool.get_ref(), id).await?.unwrap();
    mailer
        .send_registration_mail(&new_user, &registration.token)
        .await?;

    Ok(HttpResponse::Found()
        .insert_header((LOCATION, "/users"))
        .finish())
}
